import pysniffer.l4.tcp
import pysniffer.l7.textmatch
import pyprofiler.core
import logging
import netaddr
logger = logging.getLogger(__name__)

class ReportHandler:
    @staticmethod
    def register(app):
        app.singleton(ReportHandler, ReportHandler, pyprofiler.core.InformationBase)

    def __init__(self, ib):
        self.ib = ib
        #self.network = netaddr.IPNetwork('172.26.0.0/16')
        self.network = netaddr.IPNetwork('192.168.0.0/24')

        self.handlers = {}

    def boot(self):
        pass

    def registerHandler(self, report, handler):
        #print(f"DEBUG: Registered {report} handler for: {handler}")
        self.handlers[report] = handler
    
    def handleReport(self, report):
        print(f"DEBUG: handleReport is called for: {report}")
        if not report.host in self.network:
            return

        if not 'devices' in self.ib.root:
            self.ib.root['devices'] = []

        device = self.ib.root['devices'].first(host=report.host)
        #print("DEBUG: handleReport is called for device: {device}.")       
        
        #next((x for x in self.ib.root['devices'] if x['host'] == report.host), None)
        if device == None:
            device = {"host": report.host, "name": "Unknown Device", "servers": [], "clients": [], "dns-queries": []}
            self.ib.root['devices'].append(device)
            device = next((x for x in self.ib.root['devices'] if x['host'] == report.host), None)

        if report.__class__ in self.handlers:
            self.handlers[report.__class__](report, device)
            self.printReport(report)
        else:
            logger.error(f"Unhandled report: {type(report)}")

        return

    def printReport(self, report):
        print(f'{report.__module__}.{report.__class__.__name__}:')
        #print(f'{report.__dict__}')
        for key in report.__dict__:
            print(f'    {key}:')
            print(f'        {report.__dict__[key]}')
        print()
